import os
import logging
import shutil
from pathlib import Path

from tinytag import TinyTag
from pydantic import BaseModel

from jgutils.filesystem import get_files_recursive
from jgutils import converters

from .models import Song, Album, Artist

logger = logging.getLogger(__name__)

class MusicCollection(BaseModel):

    """
    Representation of a collection of songs or audio tracks
    """

    artists: list[Artist] = []

    def get_artist(self, name:str) -> Artist | None:
        for artist in self.artists:
            if artist.name == name:
                return artist
        return None

_DEFAULT_EXTENSIONS = ["flac",
                       "mp3"]
    
def read(input_dir: Path, extensions: list[str] = None) -> MusicCollection:

    """
    This function reads a music collection from a filesystem and
    uses the metadata in each song file to construct a MusicCollection.
    """

    if not extensions:
        extensions = _DEFAULT_EXTENSIONS
     
    collection = MusicCollection()
    for eachFile in get_files_recursive(input_dir, extensions):
        tag = TinyTag.get(eachFile)
        if not tag.title:
            logger.warn("Found tag with no title, skipping: %s", tag)
            continue

        artist = collection.get_artist(tag.artist)
        if not artist:
            artist = Artist(name=tag.artist)
            collection.artists.append(artist)

        if tag.album:
            album = artist.get_album(tag.album)
            if not album:
                album = Album(title=tag.album)
                artist.albums.append(album)

        song = Song(title=tag.title, identifier=str(uuid.uuid4()))
        song.formats.append(eachFile.relative_to(input_dir))
        if tag.album:
            album.songs.append(song)
        else:
            artist.songs.append(song)
        
    return collection

def refresh(collection_dir: Path, work_dir: Path):

    """
    This method refreshes a music collection by making sure each song
    has a consistently named directory and that external copies are synced
    with the reference version.
    """

    ext = "flac"
    files = get_files_recursive(collection_dir, [ext])

    for eachFile in files:
        tag = TinyTag.get(eachFile)
        if not tag.title:
            logger.warn("Found tag with no title, skipping: %s", tag)
            continue

        artist_dir = input_dir / tag.artist
        os.makedirs(artist_dir, exist_ok=True)

        if tag.album:
            os.makedirs(artist_dir / tag.album, exist_ok=True)
            shutil.move(eachFile, artist_dir / tag.album / f"{tag.title}.{ext}")
        else:
            shutil.move(eachFile, artist_dir / f"{tag.title}.{ext}")

    # Get any files not existing on the sync target, convert them to a suitable format
    # and push them to the target.
    config = converters.ConversionConfig(
        collection_dir, work_dir, "flac", "mp3"
    )
    converters.convert(config)



       
